// Panel for control buttons.
// 26 Feb 1996 Derek Lieber
//
package jd;
import  java.awt.*;
import  java.awt.event.*;
import  ui.FastPanel;
import  ui.MultiButton;
import  ui.MultiLabel;

class ButtonPanel extends FastPanel // hmm. must be Panel, not Container, otherwise flyover help windows are jittery (JDK 1.1.4)
   implements ActionListener, MouseListener
   {
   Debugger    debugger;
   
   MultiButton stopButton;
   MultiButton goButton;
   MultiButton snapshotButton;
   
   MultiButton stepOverButton;
   MultiButton stepInButton;
   MultiButton stepOutButton;
   MultiButton runButton;
   
   MultiButton editButton;

   synchronized
   void
   enableAllButtons()
      {
      stopButton.setEnabled(true);
      goButton.setEnabled(true);
      snapshotButton.setEnabled(true);
   
      stepOverButton.setEnabled(true);
      stepInButton.setEnabled(true);
      stepOutButton.setEnabled(true);
      runButton.setEnabled(true);
   
      editButton.setEnabled(true);
      }
      
   synchronized
   void
   disableAllButtons()
      {
      stopButton.setEnabled(false);
      goButton.setEnabled(false);
      snapshotButton.setEnabled(false);
   
      stepOverButton.setEnabled(false);
      stepInButton.setEnabled(false);
      stepOutButton.setEnabled(false);
      runButton.setEnabled(false);
   
      editButton.setEnabled(false);
      }
      
   ButtonPanel(Debugger debugger)
      {
      this.debugger = debugger;

      setLayout(new FlowLayout());

      add(stopButton = new MultiButton(DebuggerFrame.stopButtonImage, "Stop all threads"));
      stopButton.setEnabled(false);
      stopButton.addActionListener(this);
      stopButton.addMouseListener(this);
      
      add(goButton = new MultiButton(DebuggerFrame.goButtonImage, "Release all threads"));
      goButton.setEnabled(false);
      goButton.addActionListener(this);
      goButton.addMouseListener(this);

      add(snapshotButton = new MultiButton(DebuggerFrame.refreshButtonImage, "Refresh views"));
      snapshotButton.setEnabled(false);
      snapshotButton.addActionListener(this);
      snapshotButton.addMouseListener(this);
      
      add(new MultiLabel(null, new Insets(0, 0, 0, 2)));

      add(stepOverButton = new MultiButton(DebuggerFrame.stepOverButtonImage, "Step thread over"));
      stepOverButton.setEnabled(false);
      stepOverButton.addActionListener(this);
      stepOverButton.addMouseListener(this);
      
      add(stepInButton = new MultiButton(DebuggerFrame.stepInButtonImage, "Step thread in"));
      stepInButton.setEnabled(false);
      stepInButton.addActionListener(this);
      stepInButton.addMouseListener(this);
      
      add(stepOutButton = new MultiButton(DebuggerFrame.stepOutButtonImage, "Step thread out"));
      stepOutButton.setEnabled(false);
      stepOutButton.addActionListener(this);
      stepOutButton.addMouseListener(this);
      
      add(runButton = new MultiButton(DebuggerFrame.runButtonImage, "Run, ignoring events in other threads"));
      runButton.setEnabled(false);
      runButton.addActionListener(this);
      runButton.addMouseListener(this);
      
      add(new MultiLabel(null, new Insets(0, 0, 0, 2)));

      add(editButton = new MultiButton(DebuggerFrame.editButtonImage, "Edit source"));
      editButton.setEnabled(false);
      editButton.addActionListener(this);
      editButton.addMouseListener(this);
      }

   synchronized
   public void 
   actionPerformed(ActionEvent e) //- implements ActionListener
      {
           if (e.getSource() == stopButton)       Session.eventQueue.put(new DebuggerEvent(DebuggerEvent.BUTTON_STOP));
      else if (e.getSource() == goButton)         Session.eventQueue.put(new DebuggerEvent(DebuggerEvent.BUTTON_GO));
      else if (e.getSource() == snapshotButton)   Session.eventQueue.put(new DebuggerEvent(DebuggerEvent.BUTTON_SNAPSHOT));
      
      else if (e.getSource() == stepOverButton)   Session.eventQueue.put(new DebuggerEvent(DebuggerEvent.BUTTON_STEP_OVER));
      else if (e.getSource() == stepInButton)     Session.eventQueue.put(new DebuggerEvent(DebuggerEvent.BUTTON_STEP_IN));
      else if (e.getSource() == stepOutButton)    Session.eventQueue.put(new DebuggerEvent(DebuggerEvent.BUTTON_STEP_OUT));
      
      else if (e.getSource() == runButton)        Session.eventQueue.put(new DebuggerEvent(DebuggerEvent.BUTTON_RUN));
      else if (e.getSource() == editButton)       Session.eventQueue.put(new DebuggerEvent(DebuggerEvent.BUTTON_EDIT));
      }

   synchronized public void mouseClicked(MouseEvent e)  {            } //- implements MouseListener
   synchronized public void mouseEntered(MouseEvent e)  { flyOn(e);  } //- implements MouseListener
   synchronized public void mouseExited(MouseEvent e)   { flyOff(e); } //- implements MouseListener
   synchronized public void mousePressed(MouseEvent e)  {            } //- implements MouseListener
   synchronized public void mouseReleased(MouseEvent e) {            } //- implements MouseListener
   
   // Display flyover text for currently moused button.
   //
   synchronized
   private void
   flyOn(MouseEvent e)
      {
      MultiButton button  = (MultiButton)e.getSource();
      String      message = button.getDescription();
      Point       screenL = button.getLocationOnScreen();
      int         screenX = screenL.x + button.getSize().width / 2; // center of button
      int         screenY = screenL.y - 5;                          // top edge of button, raised a little
      Session.flyOn(message, screenX, screenY);
      }
   
   // Remove flyover text.
   //
   synchronized
   private void
   flyOff(MouseEvent e)
      {
      Session.flyOff();
      }
   }
