// Snapshot of execution stack for a remote thread.
// 11 Mar 1996 Derek Lieber
//
package jd;
import  sun.tools.debug.*;
import  util.Tracer;

class FramesSnapshot  extends Snapshot
   {
   private RemoteStackFrame remoteFrames[]; // frames to be displayed
   
   // This snapshot's currently selected stackframe (null --> no stack or no selection).
   //
   synchronized
   RemoteStackFrame
   currentSelection()
      {
      return selectedLino == -1 ? null : remoteFrames[selectedLino];
      }

   // This snapshot's "return frame" (null --> no stack or no return frame).
   //
   synchronized
   RemoteStackFrame
   returnFrame()
      {
      return remoteFrames.length > 1 ? remoteFrames[1] : null;
      }
   
   // Create snapshot of execution stack for remote thread (possibly null)
   // with specified frame (iff non-null) as "currentSelection".
   // Runs in "Debugger" thread.
   //
   FramesSnapshot(RemoteThread remoteThread, RemoteStackFrame initialSelection)
      throws Exception
      {
   // Tracer.trace("genText", "frames");

      if (remoteThread == null)
         remoteFrames = new RemoteStackFrame[0];
      else
         remoteFrames = remoteThread.dumpStack();
         
      selectedLino = -1;
      StringBuffer buf = new StringBuffer();
      for (int i = 0, n = remoteFrames.length; i < n; ++i)
         {
         RemoteStackFrame frame = remoteFrames[i];

         //!!TODO:this form of identity comparison is not quite right: it won't distinguish multiple frames from same method
         if (selectedLino == -1 &&
             initialSelection != null &&
             initialSelection.getRemoteClass().getName().equals(frame.getRemoteClass().getName()) &&
             initialSelection.getMethodName().equals(frame.getMethodName()))
            selectedLino = i;

         buf.append(frame.getRemoteClass().getName() + "." + frame.getMethodName() + "\n");
         }
      
      // if initial selection not present, default to innermost stackframe
      if (selectedLino == -1 && remoteFrames.length != 0)
         selectedLino = 0;

      text = buf.toString();
      }
      
   // Create a copy of "oldSnapshot", but with new selection at specified line.
   //
   FramesSnapshot(FramesSnapshot oldSnapshot, int lino)
      throws Exception
      {
      text         = oldSnapshot.text;
      selectedLino = lino;
      remoteFrames = oldSnapshot.remoteFrames;
      }
   }
