// Snapshot of local variables for a remote stackframe.
// 11 Mar 1996 Derek Lieber
//
package jd;
import  sun.tools.debug.*;
import  util.Tracer;

class LocalsSnapshot  extends Snapshot
   {
   private String              name;           // name of this stackframe (== className.methodName)
   private RemoteStackVariable remoteLocals[]; // variables to be displayed
   
   // This snapshot's currently selected variable (null --> no selection).
   //
   synchronized
   RemoteStackVariable
   currentSelection()
      {
      return selectedLino == -1 ? null : remoteLocals[selectedLino];
      }

   // Create snapshot of local variables for remote stackframe (possibly null),
   // with specified variable (iff non-null) as "current selection".
   // Runs in "Debugger" thread.
   //
   LocalsSnapshot(RemoteStackFrame remoteFrame, RemoteStackVariable initialSelection)
      throws Exception
      {
   // Tracer.trace("genText", "locals");
      
      if (remoteFrame == null)
         {
         name         = "<no locals>";
         remoteLocals = new RemoteStackVariable[0];
         }
      else
         {
         name         = remoteFrame.getRemoteClass().getName() + "." + remoteFrame.getMethodName();
         remoteLocals = remoteFrame.getLocalVariables();
         }
         
      // extract "live" locals from list
      //
      int cnt = 0;
      for (int i = 0, n = remoteLocals.length; i < n; ++i)
         {
         RemoteStackVariable var = remoteLocals[i];
         if (var.inScope())
            ++cnt;
         }
         
      RemoteStackVariable subList[] = new RemoteStackVariable[cnt];
      cnt = 0;
      for (int i = 0, n = remoteLocals.length; i < n; ++i)
         {
         RemoteStackVariable var = remoteLocals[i];
         if (var.inScope())
            subList[cnt++] = var;
         }

      remoteLocals = subList;
      
      // generate text corresponding to variables list
      //
      selectedLino   = -1;
      StringBuffer buf = new StringBuffer();
      for (int i = 0, n = remoteLocals.length; i < n; ++i)
         {
         RemoteStackVariable var = remoteLocals[i];

         String      varName  = var.getName();
         RemoteValue varValue = var.getValue();

         if (selectedLino   == -1 &&
             initialSelection != null && 
             initialSelection.getName().equals(varName))
            selectedLino = i;

         ValueTree.printValue(buf, null, varName, varValue);
         }

      text = buf.toString();
      }

   // Create a copy of "oldSnapshot", but with new selection at specified line.
   //
   LocalsSnapshot(LocalsSnapshot oldSnapshot, int lino)
      throws Exception
      {
      text         = oldSnapshot.text;
      selectedLino = lino;
      name         = oldSnapshot.name;
      remoteLocals = oldSnapshot.remoteLocals;
      }

   // Should "this" snapshot be displayed with same "currently visible line" as "previousSnapshot"?
   //
   synchronized
   boolean
   sameViewpointAs(Snapshot previousSnapshot) //- overrides Snapshot
      {
      LocalsSnapshot s = (LocalsSnapshot)previousSnapshot;
      return s != null && this.name.equals(s.name);
      }
   }
