// A multi-paned java debugger based on Sun's RemoteDebugger class.
// 23 Feb 1996 Derek Lieber
//
package jd;
import java.util.Properties;

public class Main
   implements Constants
   {
   public static void
   main(String args[])
      {
      System.out.println("jd: running jd version " + VERSION
                      + " using java version " + System.getProperties().get("java.version") 
                      + " on " + System.getProperties().get("os.name")
                      + " version " + System.getProperties().get("os.version")
                      );

      boolean checkVersion        = true;  // check jdk version number before starting up?
      String  classPath           = null;  // places to look for .class files
      String  sourcePath          = null;  // places to look for .java files
      String  remoteHost          = null;  // remote host to attach to (or null)
      String  remotePassword      = null;  // password for attaching to remote host (or null)
      String  vmStartupArgs       = "";    // args with which to start debugee vm
      String  threadStartupArgs[] = null;  // args with which to start debugee thread
      
      // gather two sets of arguments: one set for debuggee (remote), one set for debugger (local)
      //
      for (int i = 0; i < args.length; i++)
         {
         String arg = args[i];
         
         if (arg.equals("-cs")           || // -cs            check if source is newer when loading classes
             arg.equals("-checksource")  || // -checksource   check if source is newer when loading classes
             arg.equals("-noasyncgc")    || // -noasyncgc     don't allow asynchronous gc's
             arg.equals("-prof")         || // -prof          output profiling data to ./java.prof
             arg.equals("-v")            || // -v             turn on verbose mode
             arg.equals("-verbose")      || // -verbose       turn on verbose mode
             arg.equals("-verify")       || // -verify        verify all classes when read in
             arg.equals("-noverify")     || // -noverify      do not verify any class
             arg.equals("-verifyremote") || // -verifyremote  verify classes read in over the network [default]
             arg.equals("-verbosegc")    || // -verbosegc     print a message when GCs occur
             arg.startsWith("-ms")       || // -ms<number>    set the initial Java heap size
             arg.startsWith("-mx")       || // -mx<number>    set the maximum Java heap size
             arg.startsWith("-ss")       || // -ss<number>    set the C stack size of a process
             arg.startsWith("-oss")      || // -oss<number>   set the JAVA stack size of a process
             arg.startsWith("-nojit")    || // -nojit         disable JIT compiler
             arg.startsWith("-D"))          // -Dxxx=yyy      define a property value
            {
            vmStartupArgs += arg + " ";
            continue;
            }
         
         if (arg.equals("-noversioncheck")) // -noversioncheck
            {
            checkVersion = false;
            continue;
            }

         if (arg.equals("-classpath"))      // -classpath <list of directories>
            {
            if (i == args.length - 1)
               {
               System.out.println("jd: missing argument for -classpath.");
               System.exit(1);
               }
            classPath = args[++i];
            continue;
            }

         if (arg.equals("-sourcepath"))      // -sourcepath <list of directories>
            {
            if (i == args.length - 1)
               {
               System.out.println("jd: missing argument for -sourcepath.");
               System.exit(1);
               }
            sourcePath = args[++i];
            continue;
            }

         if (arg.equals("-host"))           // -host <host>
            {
            if (i == args.length - 1)
               {
               System.out.println("jd: missing argument for -host.");
               System.exit(1);
               }
            remoteHost = args[++i];
            continue;
            }

         if (arg.equals("-password"))       // -password <password>
            {
            if (i == args.length - 1)
               {
               System.out.println("jd: missing argument for -password.");
               System.exit(1);
               }
            remotePassword = args[++i];
            continue;
            }
       
         if (arg.equals("-help"))
            {
            System.out.println("jd: usage:  jd [jd_args] [jvm_args] [class_name [class_args]]");
            System.out.println("       or:  jd [jd_args] -host hostname -password password");
            System.out.println("  jd_args:  [-noversioncheck][-classpath classpath][-sourcepath sourcepath]");
            System.out.println("  jvm_args: type \"java -help\" for a list");
            System.exit(1);
            }

         if (arg.startsWith("-"))
            {
            System.out.println("jd: unrecognized flag: " + arg);
            System.exit(1);
            }

         int cnt = args.length - i;
         if (cnt != 0)
            {
            threadStartupArgs = new String[cnt];
            for (int j = 0; j < cnt; )
               threadStartupArgs[j++] = args[i++];
            }
         break;
         }

      if (checkVersion && !System.getProperties().get("java.version").equals("1.1.6"))
         {
         System.out.println("jd: sorry, this version of jd only works with java version 1.1.6");
         System.out.println("jd: use \"-noversioncheck\" if you want to proceed anyway");
         System.exit(1);
         }

      if (classPath == null)
         classPath = System.getProperty("java.class.path");
      
      if (sourcePath == null)
         sourcePath = classPath + System.getProperty("path.separator") + System.getProperty("java.home") + System.getProperty("file.separator") + "src";
      
      if (remoteHost != null)
         { // attach to already-running VM
         if (remotePassword == null)
            {
            System.out.println("jd: missing -password argument.");
            System.exit(1);
            }
         new Debugger(classPath, sourcePath, true, remoteHost, remotePassword, null, null).start();
         }
      else
         { // run debugee in a new VM
         new Debugger(classPath, sourcePath, false, null, null, vmStartupArgs, threadStartupArgs).start();
         }
      }
   }
