// Present a message string within a modal popup window.
// See also: PathDialog
// 27 Aug 1997 Derek Lieber
//
package jd;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import ui.*;

class MessageDialog extends Dialog
   {
   private Frame        frame;
   private TextScroller textScroller;
   private Container    buttonPanel;
   private boolean      result;

   // Create message window.
   // Taken: frame relative to which we should pop up
   //        string to display in title bar
   //        string to display in text area
   //
   MessageDialog(Frame _frame, String title, String text)
      {
      super(_frame, title, true);

      frame        = _frame;
      textScroller =  new TextScroller();
      buttonPanel  =  new Panel();

      setLayout(new BorderLayout());
      add("Center", textScroller);
      add("South",  buttonPanel);
      
      MultiButton okButton = new MultiButton(" Ok ", null);
      
      okButton.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { ok(); }});
      addWindowListener(new WindowAdapter() { public void windowClosing(WindowEvent e) { cancel(); }});

      buttonPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
      buttonPanel.add("", okButton);
      
      textScroller.setText(text + "\n");
      textScroller.setEdging(TextScroller.NO_EDGING);
      }

   // Allow message to be viewed.
   // Taken:    nothing
   // Returned: was message acknowledged with "ok" button?
   // Note: to avoid a deadlock between Dialog.show() and Dialog.ok(), this method must NOT be synchronized!
   //
// synchronized
   boolean
   edit()
      {
      pack();

      Point     frameLoc   = frame.getLocationOnScreen();
      Dimension frameSize  = frame.getSize();
      Dimension dialogSize = getPreferredSize();
      
      int x = frameLoc.x + frameSize.width  / 2 - dialogSize.width  / 2;
      int y = frameLoc.y + frameSize.height / 2 - dialogSize.height / 2;
      
      if (x < 0) x = 0;
      if (y < 0) y = 0;

      setBounds(x, y, dialogSize.width, dialogSize.height);
      show(); // note: dialog is "modal", meaning that control blocks here until ok() or cancel() hides the window again
      dispose();

      return result;
      }

   synchronized
   void
   ok()
      {
      result = true;
      
      // AWT bug workaround: prevent desktop from getting focus when dialog vanishes
      //
      frame.toFront();      
      Thread.currentThread().yield();
      
      // unblock caller at site of "dialog.show()"
      //
      setVisible(false);
      }

   synchronized
   void
   cancel()
      {
      result = false;
      
      // AWT bug workaround: prevent desktop from getting focus when dialog vanishes
      //
      frame.toFront();      
      Thread.currentThread().yield();
      
      // unblock caller at site of "dialog.show()"
      //
      setVisible(false);
      }
   }
