// The standard AWT library generates unnecessary paint() requests as a side effect
// of setBounds(), add(), and remove(). This leads to excessive screen redrawing.
//
// This "fast" panel class suppresses such behavior, in order to orthogonalize
// the operations of window shaping vs. drawing. The advantage is better screen
// performance. The disadvantage is that subclasses must decide for themselves when 
// to redraw the screen.
//
// See also: FastContainer, FastComponent
//
// 24 Sep 1997 Derek Lieber
//
package ui;
import java.awt.*;

public class FastPanel extends Panel
   {
   public void
   update(Graphics g) //- overrides Container
      {
   // System.out.println("ignoring panel update on " + getName());
      }

   public void
   repaint(long tm, int x, int y, int width, int height) //- overrides Component
      {
   // System.out.println("ignoring panel repaint on " + getName());
      }
   }
