// Utility methods for ui components.
//
package ui;
import  java.awt.*;

public
class Util
   {
   // Get bounds of a component, in its *local* coordinate system.
   // See also "Component.getBounds()", which returns bounds in *container* coordinate system.
   //
   public static Rectangle
   getLocalBounds(Component c)
      {
      Dimension d = c.getSize();
      return new Rectangle(0, 0, d.width, d.height);
      }

   // Render a component "immediately", without waiting for asynchronous paint().
   //
   public static void
   redraw(Component c)
      {
      Graphics g = c.getGraphics();
      if (g == null)
         return; // not yet, or no longer, backed by a native window (probably window was closed)

      try { c.paint(g); } finally { g.dispose(); }
      }
      
   // Unrender a component, by drawing its area in the background color of its container.
   //
   public static void
   undraw(Component c)
      {
      Graphics g = c.getGraphics();
      if (g == null)
         return; // not yet, or no longer, backed by a native window (probably window was closed)

      g.setColor(c.getParent().getBackground());
      g.fillRect(0, 0, c.getSize().width, c.getSize().height);
      g.dispose();
      }
      
   // Render beveled border with "pushed in" look.
   //
   public static void
   drawInsetBorder(Graphics g, Rectangle bounds)
      {
      drawBorder(g, bounds,
                    SystemColor.controlDkShadow,
                    SystemColor.controlShadow,
                    SystemColor.controlHighlight,
                    SystemColor.controlLtHighlight);
      }
   
   // Render beveled border with "pushed out" look.
   //
   public static void
   drawOutsetBorder(Graphics g, Rectangle bounds)
      {
      drawBorder(g, bounds,
                    SystemColor.controlLtHighlight,
                    SystemColor.controlHighlight,
                    SystemColor.controlShadow,
                    SystemColor.controlDkShadow);
      }

   // Render beveled border.
   // Assumption: BEVEL_THICKNESS == 2.
   //
   private static void
   drawBorder(Graphics g, Rectangle bounds, Color topLeftOuter,     
                                            Color topLeftInner,
                                            Color bottomRightInner, 
                                            Color bottomRightOuter)
      {
      int x = bounds.x;
      int y = bounds.y;
      int w = bounds.width;
      int h = bounds.height;
      
      g.setColor(topLeftOuter);
      g.drawLine(x, y, x + w - 1, y);
      g.drawLine(x, y, x, y + h - 1);
      
      g.setColor(topLeftInner);
      g.drawLine(x + 1, y + 1, x + w - 2, y + 1);
      g.drawLine(x + 1, y + 1, x + 1,     y + h - 2);
      
      g.setColor(bottomRightInner);
      g.drawLine(x + 1, y + h - 2, x + w - 2, y + h - 2);
      g.drawLine(x + w - 2, y + 1, x + w - 2, y + h - 2);
      
      g.setColor(bottomRightOuter);
      g.drawLine(x + w - 1, y, x + w - 1, y + h - 1);
      g.drawLine(x, y + h - 1, x + w - 1, y + h - 1);
      }
   }
